@@ -26,15 +26,6 @@
 #include "babl-db.h"
 #include "babl-ref-pixels.h"
 
-static int model_is_rgba (const Babl *model)
-{
-  const Babl *RGBA = babl_model_from_id (BABL_RGBA);
-  if (model == RGBA || model->model.model == RGBA)
-    return 1;
-  return 0;
-}
-
-
 static void
 babl_conversion_plane_process (BablConversion *conversion,
                                const void     *source,
@@ -150,7 +141,8 @@ _conversion_new (const char    *name,
                  BablFuncLinear linear,
                  BablFuncPlane  plane,
                  BablFuncPlanar planar,
-                 void          *user_data)
+                 void          *user_data,
+                 int            allow_collision)
 {
   Babl *babl = NULL;
 
@@ -235,11 +227,17 @@ _conversion_new (const char    *name,
         BABL (babl->conversion.destination),
         babl_type_from_id (BABL_DOUBLE));
 
+      if(allow_collision){
+        const Babl *fish = babl_conversion_find (src_format, dst_format);
+        if (fish)
+          return (void*)fish;
+      }
       babl_conversion_new (
         src_format,
         dst_format,
         "linear", linear,
         "data", user_data,
+        allow_collision?"allow-collision":NULL,
         NULL);
       babl->conversion.error = 0.0;
     }
@@ -276,17 +274,19 @@ create_name (Babl *source, Babl *destination, int type)
     }
   return buf;
 }
-const char *
-babl_conversion_create_name (Babl *source, Babl *destination, int type);
 
 const char *
-babl_conversion_create_name (Babl *source, Babl *destination, int type)
+babl_conversion_create_name (Babl *source, Babl *destination, int type,
+                             int allow_collision)
 {
   Babl *babl;
   char *name;
   int id = 0;
   collisions = 0;
   name = create_name (source, destination, type);
+
+  if (allow_collision == 0)
+  {
   babl = babl_db_exist (db, id, name);
   while (babl)
     {
@@ -297,6 +297,7 @@ babl_conversion_create_name (Babl *source, Babl *destination, int type)
       name = create_name (source, destination, type);
       babl = babl_db_exist (db, id, name);
     }
+  }
   return name;
 }
 
@@ -318,8 +319,8 @@ babl_conversion_new (const void *first_arg,
 
   Babl          *source;
   Babl          *destination;
-
   char          *name;
+  int            allow_collision = 0;
 
   va_start (varg, first_arg);
   source      = (Babl *) arg;
@@ -342,6 +343,10 @@ babl_conversion_new (const void *first_arg,
           user_data = va_arg (varg, void*);
         }
 
+      else if (!strcmp (arg, "allow-collision"))
+        {
+          allow_collision = 1;
+        }
       else if (!strcmp (arg, "linear"))
         {
           if (got_func++)
@@ -395,10 +400,10 @@ babl_conversion_new (const void *first_arg,
       type = BABL_CONVERSION_PLANAR;
     }
 
-  name = (void*) babl_conversion_create_name (source, destination, type);
+  name = (void*) babl_conversion_create_name (source, destination, type, allow_collision);
 
   babl = _conversion_new (name, id, source, destination, linear, plane, planar,
-                         user_data);
+                          user_data, allow_collision);
 
   /* Since there is not an already registered instance by the required
    * id/name, inserting newly created class into database.
