@@ -18,6 +18,10 @@
 
 #include "config.h"
 #include "babl-internal.h"
+#ifdef HAVE_LCMS
+#include "lcms2.h"
+#endif
+
 
 static Babl *
 assert_conversion_find (const void *source,
@@ -28,6 +32,7 @@ assert_conversion_find (const void *source,
   if (!ret)
     babl_fatal ("failed finding conversion between %s and %s aborting",
                 babl_get_name (source), babl_get_name (destination));
+
   return ret;
 }
 
@@ -231,7 +236,8 @@ convert_to_double (BablFormat      *source_fmt,
 
 
 static void
-convert_from_double (BablFormat *destination_fmt,
+convert_from_double (BablFormat *source_fmt,
+                     BablFormat *destination_fmt,
                      char       *destination_double_buf,
                      char       *destination_buf,
                      int         n)
@@ -251,16 +257,33 @@ convert_from_double (BablFormat *destination_fmt,
   src_img->stride[0] = 0;
 
   dst_img->data[0]  = destination_buf;
-  dst_img->type[0]  = (BablType *) babl_type_from_id (BABL_DOUBLE);
   dst_img->pitch[0] = destination_fmt->bytes_per_pixel;
   dst_img->stride[0] = 0;
 
   for (i = 0; i < destination_fmt->components; i++)
     {
       int j;
+      int can_be_used = 1;
 
       dst_img->type[0] = destination_fmt->type[i];
 
+      if (source_fmt->model == destination_fmt->model)
+      {
+        can_be_used = 0;
+        for (j = 0; j < source_fmt->components; j++)
+        {
+          if (destination_fmt->component[i] == source_fmt->component[j])
+          {
+            can_be_used = 1;
+          }
+        }
+      }
+      else
+      {
+      }
+         //fprintf (stderr, "%s %s %i\n", babl_get_name (source_fmt), babl_get_name (destination_fmt), can_be_used);
+
+      if (can_be_used)
       for (j = 0; j < destination_fmt->model->components; j++)
         {
           if (destination_fmt->component[i] ==
@@ -438,7 +461,8 @@ ncomponent_convert_to_float (BablFormat       *source_fmt,
 }
 
 static void
-ncomponent_convert_from_float (BablFormat *destination_fmt,
+ncomponent_convert_from_float (BablFormat *source_fmt,
+                               BablFormat *destination_fmt,
                                char       *destination_float_buf,
                                char       *destination_buf,
                                int         n)
@@ -550,7 +574,8 @@ convert_to_float (BablFormat *source_fmt,
 
 
 static void
-convert_from_float (BablFormat *destination_fmt,
+convert_from_float (BablFormat *source_fmt,
+                    BablFormat *destination_fmt,
                      char      *destination_float_buf,
                      char      *destination_buf,
                      int        n)
@@ -577,9 +602,23 @@ convert_from_float (BablFormat *destination_fmt,
   for (i = 0; i < destination_fmt->components; i++)
     {
       int j;
+      int can_be_used = 1;
 
       dst_img->type[0] = destination_fmt->type[i];
 
+      if (source_fmt->model == destination_fmt->model)
+      {
+        can_be_used = 0;
+        for (j = 0; j < source_fmt->components; j++)
+        {
+          if (destination_fmt->component[i] == source_fmt->component[j])
+          {
+            can_be_used = 1;
+          }
+        }
+      }
+
+      if (can_be_used)
       for (j = 0; j < destination_fmt->model->components; j++)
         {
           if (destination_fmt->component[i] ==
@@ -612,8 +651,6 @@ process_same_model (const Babl  *babl,
   const void *type_float = babl_type_from_id (BABL_FLOAT);
 #define MAX(a, b) ((a) > (b) ? (a) : (b))
 
-
-
   if ((babl->fish.source->format.type[0]->bits < 32 ||
        babl->fish.source->format.type[0] == type_float) &&
       (babl->fish.destination->format.type[0]->bits < 32 ||
@@ -631,6 +668,7 @@ process_same_model (const Babl  *babl,
           float_buf,
           n);
         ncomponent_convert_from_float (
+          (BablFormat *) BABL (babl->fish.source),
           (BablFormat *) BABL (babl->fish.destination),
           float_buf,
           (char *) destination,
@@ -645,6 +683,7 @@ process_same_model (const Babl  *babl,
           n);
 
         convert_from_float (
+          (BablFormat *) BABL (babl->fish.source),
           (BablFormat *) BABL (babl->fish.destination),
           float_buf,
           (char *) destination,
@@ -681,6 +720,7 @@ process_same_model (const Babl  *babl,
           n);
 
         convert_from_double (
+          (BablFormat *) BABL (babl->fish.source),
           (BablFormat *) BABL (babl->fish.destination),
           double_buf,
           (char *) destination,
@@ -690,6 +730,13 @@ process_same_model (const Babl  *babl,
   }
 }
 
+typedef enum _Kind Kind;
+enum _Kind { KIND_RGB, KIND_CMYK};
+
+static int format_has_cmyk_model (const Babl *format)
+{
+  return format->format.model->flags & BABL_MODEL_FLAG_CMYK;
+}
 
 static void
 babl_fish_reference_process_double (const Babl *babl,
@@ -698,154 +745,453 @@ babl_fish_reference_process_double (const Babl *babl,
                                     long        n,
                                     void       *data)
 {
-    Babl *source_image = NULL;
-    Babl *rgba_image = NULL;
-    Babl *destination_image = NULL;
-    void *source_double_buf_alloc = NULL;
-    void *source_double_buf;
-    void *rgba_double_buf_alloc = NULL;
-    void *rgba_double_buf;
-    void *destination_double_buf_alloc = NULL;
-    void *destination_double_buf;
-    const void *type_double  = babl_type_from_id (BABL_DOUBLE);
+  Kind  source_kind             = KIND_RGB;
+  Kind  destination_kind        = KIND_RGB;
+  Babl *source_image            = NULL;
+  Babl *rgba_image              = NULL;
+  Babl *cmyka_image             = NULL;
+  Babl *destination_image       = NULL;
 
-    if (babl->fish.source->format.type[0] == type_double &&
-        BABL(babl->fish.source)->format.components ==
-        BABL(babl->fish.source)->format.model->components && 0)
+
+
+  void *source_double_buf_alloc = NULL;
+  void *source_double_buf;
+  void *rgba_double_buf_alloc   = NULL;
+  void *rgba_double_buf;
+  void *cmyka_double_buf_alloc  = NULL;
+  void *cmyka_double_buf;
+  void *destination_double_buf_alloc = NULL;
+  void *destination_double_buf;
+  const void *type_double  = babl_type_from_id (BABL_DOUBLE);
+
+  const Babl *source_space = BABL (BABL ((babl->fish.source))->format.space);
+  const Babl *destination_space = BABL (BABL ((babl->fish.destination))->format.space);
+
+  /* This is not the full/only condition XXX */
+
+  /* XXX : sometimes is_cmyk is neither 0 or 1 */
+
+  if (format_has_cmyk_model (babl->fish.source))
+    source_kind = KIND_CMYK;
+  if (format_has_cmyk_model (babl->fish.destination))
+    destination_kind = KIND_CMYK;
+
+  if (babl->fish.source->format.type[0] == type_double &&
+      BABL(babl->fish.source)->format.components ==
+      BABL(babl->fish.source)->format.model->components)
+  {
+    source_double_buf = (void*)source;
+    source_image = babl_image_from_linear (
+       source_double_buf, BABL (BABL ((babl->fish.source))->format.model));
+  }
+  else
+  {
+    source_double_buf =
+    source_double_buf_alloc = babl_malloc (sizeof (double) * n *
+                                BABL (babl->fish.source)->format.model->components);
+
+    source_image = babl_image_from_linear (
+      source_double_buf, BABL (BABL ((babl->fish.source))->format.model));
+    convert_to_double (
+      (BablFormat *) BABL (babl->fish.source),
+      source,
+      source_double_buf,
+      n
+    );
+  }
+
+  babl_mutex_lock (babl_reference_mutex);
+  switch (source_kind)
+  {
+    case KIND_RGB:
+     {
+       Babl *conv = assert_conversion_find (
+           BABL (babl->fish.source)->format.model,
+       babl_remodel_with_space (babl_model_from_id (BABL_RGBA),
+                         source_space));
+
+       rgba_double_buf       =
+       rgba_double_buf_alloc = babl_malloc (sizeof (double) * n * 4);
+
+       rgba_image = babl_image_from_linear (
+          rgba_double_buf, babl_remodel_with_space (babl_model_from_id (BABL_RGBA), source_space));
+
+       if (conv->class_type == BABL_CONVERSION_PLANAR)
+       {
+          babl_conversion_process (conv,
+               (void*)source_image, (void*)rgba_image, n);
+       }
+       else if (conv->class_type == BABL_CONVERSION_LINEAR)
+       {
+         babl_conversion_process (conv, source_double_buf, rgba_double_buf, n);
+       }
+       else babl_fatal ("oops");
+      }
+      break;
+    case KIND_CMYK:
+      if (babl_model_is ((void*)babl->fish.source->format.model, "cmykA"))
+      {
+        cmyka_double_buf = source_double_buf;
+        cmyka_image = babl_image_from_linear (cmyka_double_buf,
+                                (void*)babl->fish.source->format.model);
+      }
+      else
+      {
+        Babl *conv = assert_conversion_find (
+                   BABL (babl->fish.source)->format.model,
+                   babl_remodel_with_space (babl_model ("cmykA"),
+                         source_space));
+
+        cmyka_double_buf       =
+        cmyka_double_buf_alloc = babl_malloc (sizeof (double) * n * 5);
+
+        cmyka_image = babl_image_from_linear (
+          cmyka_double_buf, babl_remodel_with_space (babl_model ("cmykA"),
+          source_space));
+
+        if (conv->class_type == BABL_CONVERSION_PLANAR)
+        {
+          babl_conversion_process (conv,
+           (void*)source_image, (void*)cmyka_image, n);
+        }
+        else if (conv->class_type == BABL_CONVERSION_LINEAR)
+        {
+          babl_conversion_process (conv,source_double_buf, cmyka_double_buf, n);
+        }
+        else babl_fatal ("oops");
+      }
+      break;
+  }
+
+  if (source_kind      == KIND_RGB &&
+      destination_kind == KIND_RGB)
+  {
+    if (source_space != destination_space)
     {
-      source_double_buf = (void*)source;
-      source_image = babl_image_from_linear (
-         source_double_buf, BABL (BABL ((babl->fish.source))->format.model));
+      double matrix[9];
+      double *rgba = rgba_double_buf;
+      babl_matrix_mul_matrix (
+        destination_space->space.XYZtoRGB,
+        source_space->space.RGBtoXYZ,
+        matrix);
+
+      babl_matrix_mul_vector_buf4 (matrix, rgba, rgba, n);
+    }
+  }
+  else if (source_kind      == KIND_RGB &&
+           destination_kind == KIND_CMYK)
+  {
+    cmyka_double_buf        =
+    cmyka_double_buf_alloc  = babl_malloc (sizeof (double) * n * 5);
+    cmyka_image = babl_image_from_linear (
+        cmyka_double_buf, babl_remodel_with_space (babl_model ("cmykA"),
+        destination_space));
+
+#if HAVE_LCMS
+    if (destination_space->space.cmyk.lcms_profile)
+    {
+      /* lcms expect floats with normalized range 0.0-100.0 for CMYK data,
+         we also do our inversion from profile here.
+       */
+      double *rgba=rgba_double_buf;
+      double *cmyka=cmyka_double_buf;
+      int i;
+      /* use lcms for doing conversion from RGBA */
+      cmsDoTransform (destination_space->space.cmyk.lcms_from_rgba,
+         rgba_double_buf, cmyka_double_buf, n);
+
+      for (i = 0; i < n; i++)
+      {
+        cmyka[i * 5 + 0] = 1.0-(cmyka[i * 5 + 0])/100.0;
+        cmyka[i * 5 + 1] = 1.0-(cmyka[i * 5 + 1])/100.0;
+        cmyka[i * 5 + 2] = 1.0-(cmyka[i * 5 + 2])/100.0;
+        cmyka[i * 5 + 3] = 1.0-(cmyka[i * 5 + 3])/100.0;
+        cmyka[i * 5 + 4] = rgba[i * 4 + 3];
+      }
     }
     else
+#endif
     {
-      source_double_buf_alloc = babl_malloc (sizeof (double) * n *
-                                  BABL (babl->fish.source)->format.model->components);
+      double *rgba=rgba_double_buf;
+      double *cmyka=cmyka_double_buf;
+      int i;
+      for (i = 0; i < n; i++)
+      {
+        /* A very naive conversion - but it is usable */
+        double key=0.0;
+        cmyka[i * 5 + 0] = 1.0 - rgba[i * 4 + 0];
+        cmyka[i * 5 + 1] = 1.0 - rgba[i * 4 + 1];
+        cmyka[i * 5 + 2] = 1.0 - rgba[i * 4 + 2];
 
-      source_double_buf = source_double_buf_alloc;
-      source_image = babl_image_from_linear (
-        source_double_buf, BABL (BABL ((babl->fish.source))->format.model));
-      convert_to_double (
-        (BablFormat *) BABL (babl->fish.source),
-        source,
-        source_double_buf,
-        n
-      );
+        if (cmyka[i * 5 + 0] < key) key = cmyka[i*5+0];
+        if (cmyka[i * 5 + 1] < key) key = cmyka[i*5+1];
+        if (cmyka[i * 5 + 2] < key) key = cmyka[i*5+2];
+
+        key *= 1.0; // pullout - XXX tune default pullout?;
+
+        if (key < 1.0)
+        {
+          cmyka[i * 5 + 0] = (cmyka[i * 5 + 0] - key) / (1.0-key);
+          cmyka[i * 5 + 1] = (cmyka[i * 5 + 1] - key) / (1.0-key);
+          cmyka[i * 5 + 2] = (cmyka[i * 5 + 2] - key) / (1.0-key);
+        }
+        cmyka[i * 5 + 0] = 1.0-cmyka[i * 5 + 0];
+        cmyka[i * 5 + 1] = 1.0-cmyka[i * 5 + 1];
+        cmyka[i * 5 + 2] = 1.0-cmyka[i * 5 + 2];
+        cmyka[i * 5 + 3] = 1.0-key;
+        cmyka[i * 5 + 4] = rgba[i * 4 + 3];
+      }
     }
+ }
+ else if (source_kind      == KIND_CMYK &&
+          destination_kind == KIND_RGB)
+ {
+    /* */
+    rgba_double_buf_alloc  = babl_malloc (sizeof (double) * n * 4);
+    rgba_double_buf        = rgba_double_buf_alloc;
+    rgba_image = babl_image_from_linear (
+        rgba_double_buf, babl_remodel_with_space (babl_model_from_id (BABL_RGBA),
+        destination_space));
 
-    if (babl_model_is ((void*)babl->fish.source->format.model, "RGBA"))
+#if HAVE_LCMS
+    if (source_space->space.cmyk.lcms_profile)
     {
-      rgba_double_buf = source_double_buf;
-      rgba_image = babl_image_from_linear (
-          rgba_double_buf,
-          (void*)babl->fish.source->format.model);
+    {
+      /* lcms expect floats with normalized range 0.0-100.0 for CMYK data,
+         we also do our inversion from profile here.
+       */
+      double *cmyka=cmyka_double_buf;
+      int i;
+      for (i = 0; i < n; i++)
+      {
+        cmyka[i * 5 + 0] = (1.0-cmyka[i * 5 + 0])*100.0;
+        cmyka[i * 5 + 1] = (1.0-cmyka[i * 5 + 1])*100.0;
+        cmyka[i * 5 + 2] = (1.0-cmyka[i * 5 + 2])*100.0;
+        cmyka[i * 5 + 3] = (1.0-cmyka[i * 5 + 3])*100.0;
+      }
+    }
+    /* use lcms for doing conversion to RGBA */
+    cmsDoTransform (source_space->space.cmyk.lcms_to_rgba,
+       cmyka_double_buf, rgba_double_buf, n);
+
+    {
+      double *rgba=rgba_double_buf;
+      double *cmyka=cmyka_double_buf;
+      int i;
+      for (i = 0; i < n; i++)
+      {
+        rgba[i * 4 + 3] = cmyka[i * 5 + 4];
+      }
+    }
     }
     else
+#endif
     {
-      Babl *conv =
-        assert_conversion_find (
-        BABL (babl->fish.source)->format.model,
-
-      babl_remodel_with_space (babl_model_from_id (BABL_RGBA),
-                           BABL (BABL ((babl->fish.source))->format.space)));
-
-      rgba_double_buf_alloc  = babl_malloc (sizeof (double) * n * 4);
-      rgba_double_buf        = rgba_double_buf_alloc;
-
-      rgba_image = babl_image_from_linear (
-          rgba_double_buf, babl_remodel_with_space (babl_model_from_id (BABL_RGBA),
-          BABL (BABL ((babl->fish.source))->format.space)) );
-
-      if (conv->class_type == BABL_CONVERSION_PLANAR)
+      double *rgba=rgba_double_buf;
+      double *cmyka=cmyka_double_buf;
+      int i;
+      for (i = 0; i < n; i++)
       {
-        babl_conversion_process (
-          conv,
-          (void*)source_image, (void*)rgba_image,
-          n);
+        /* A very naive conversion - but it is usable */
+        rgba[i * 4 + 0] = cmyka[i * 5 + 0]*cmyka[i*5+3];
+        rgba[i * 4 + 1] = cmyka[i * 5 + 1]*cmyka[i*5+3];
+        rgba[i * 4 + 2] = cmyka[i * 5 + 2]*cmyka[i*5+3];
+        rgba[i * 4 + 3] = cmyka[i * 5 + 4];
       }
-      else if (conv->class_type == BABL_CONVERSION_LINEAR)
-      {
-        babl_conversion_process (
-          conv,
-          source_double_buf, rgba_double_buf,
-          n);
-      }
-      else babl_fatal ("oops");
     }
 
-    if (((babl->fish.source)->format.space !=
+    /* color space conversions */
+     if ((babl_space ("scRGB")!=
         ((babl->fish.destination)->format.space)))
     {
       double matrix[9];
       double *rgba = rgba_double_buf;
       babl_matrix_mul_matrix (
         (babl->fish.destination)->format.space->space.XYZtoRGB,
-        (babl->fish.source)->format.space->space.RGBtoXYZ,
+        babl_space("scRGB")->space.RGBtoXYZ,
         matrix);
 
       babl_matrix_mul_vector_buf4 (matrix, rgba, rgba, n);
     }
-
+ }
+ else if (source_kind      == KIND_CMYK &&
+          destination_kind == KIND_CMYK)
+ {
+    if (source_space != destination_space
+#if HAVE_LCMS
+        && source_space->space.cmyk.lcms_profile
+        && destination_space->space.cmyk.lcms_profile
+#endif
+ )
     {
-      const Babl *destination_rgba_format =
-        babl_remodel_with_space (babl_model_from_id (BABL_RGBA),
-             BABL (BABL ((babl->fish.destination))->format.space));
+#if HAVE_LCMS
 
-      if(BABL (babl->fish.destination)->format.model == (void*)destination_rgba_format)
+#define MAX_CMYK_CMYK  64
+     static int cmyk_cmyk_count = 0;
+     static const Babl *cmyk_cmyk_source[MAX_CMYK_CMYK];
+     static const Babl *cmyk_cmyk_destination[MAX_CMYK_CMYK];
+     static cmsHTRANSFORM cmyk_cmyk_transform[MAX_CMYK_CMYK];
+
+     int cmyk_cmyk_no;
+      double *cmyka = cmyka_double_buf;
+     for (cmyk_cmyk_no = 0; cmyk_cmyk_no < cmyk_cmyk_count; cmyk_cmyk_no++)
+     {
+       if (cmyk_cmyk_source[cmyk_cmyk_no] == source_space &&
+           cmyk_cmyk_destination[cmyk_cmyk_no] == destination_space)
+         break;
+     }
+
+/* these are not defined by lcms2.h we hope that following the existing pattern of pixel-format definitions work */
+#ifndef TYPE_CMYKA_DBL
+#define TYPE_CMYKA_DBL      (FLOAT_SH(1)|COLORSPACE_SH(PT_CMYK)|EXTRA_SH(1)|CHANNELS_SH(4)|BYTES_SH(0))
+#endif
+
+     if (cmyk_cmyk_no >= cmyk_cmyk_count)
+     {
+       cmsHPROFILE src_profile = cmsOpenProfileFromMem(source_space->space.icc_profile, source_space->space.icc_length);
+       cmsHPROFILE dst_profile = cmsOpenProfileFromMem(destination_space->space.icc_profile, source_space->space.icc_length);
+
+       cmyk_cmyk_source[cmyk_cmyk_no] = source_space;
+       cmyk_cmyk_destination[cmyk_cmyk_no] = destination_space;
+       cmyk_cmyk_transform[cmyk_cmyk_no] = cmsCreateTransform(src_profile, TYPE_CMYKA_DBL,
+  dst_profile, TYPE_CMYKA_DBL,
+  INTENT_RELATIVE_COLORIMETRIC, cmsFLAGS_BLACKPOINTCOMPENSATION);
+       cmsCloseProfile (src_profile);
+       cmsCloseProfile (dst_profile);
+
+       cmyk_cmyk_count ++;
+     }
+      for (int i = 0; i < n; i++)
       {
-         destination_double_buf = rgba_double_buf;
+        cmyka[i * 5 + 0] = (1.0-cmyka[i * 5 + 0])*100.0;
+        cmyka[i * 5 + 1] = (1.0-cmyka[i * 5 + 1])*100.0;
+        cmyka[i * 5 + 2] = (1.0-cmyka[i * 5 + 2])*100.0;
+        cmyka[i * 5 + 3] = (1.0-cmyka[i * 5 + 3])*100.0;
       }
-      else
-      {
-      Babl *conv =
-        assert_conversion_find (destination_rgba_format,
-           BABL (babl->fish.destination)->format.model);
 
-         destination_double_buf_alloc = babl_malloc (sizeof (double) * n *
+     cmsDoTransform (cmyk_cmyk_transform[cmyk_cmyk_no],
+                     cmyka_double_buf, cmyka_double_buf, n);
+
+      for (int i = 0; i < n; i++)
+      {
+        cmyka[i * 5 + 0] = 1.0-(cmyka[i * 5 + 0])/100.0;
+        cmyka[i * 5 + 1] = 1.0-(cmyka[i * 5 + 1])/100.0;
+        cmyka[i * 5 + 2] = 1.0-(cmyka[i * 5 + 2])/100.0;
+        cmyka[i * 5 + 3] = 1.0-(cmyka[i * 5 + 3])/100.0;
+      }
+
+#endif
+     }
+ }
+
+
+  switch (destination_kind) /* XXX: the cases can share logic */
+  {
+    case KIND_CMYK:
+      {
+        const Babl *destination_cmyka_format =
+          babl_remodel_with_space (babl_model ("cmykA"), destination_space);
+        if(BABL (babl->fish.destination)->format.model == (void*)destination_cmyka_format)
+        {
+           destination_double_buf = cmyka_double_buf;
+        }
+        else
+        {
+          Babl *conv =
+            assert_conversion_find (destination_cmyka_format,
+                             BABL (babl->fish.destination)->format.model);
+          destination_double_buf =
+          destination_double_buf_alloc = babl_malloc (sizeof (double) * n *
                                           BABL (babl->fish.destination)->format.model->components);
-         destination_double_buf = destination_double_buf_alloc;
-
-      if (conv->class_type == BABL_CONVERSION_PLANAR)
-        {
-          destination_image = babl_image_from_linear (
-            destination_double_buf, BABL (BABL ((babl->fish.destination))->format.model));
-
-
-          babl_conversion_process (
-            conv,
-            (void*)rgba_image, (void*)destination_image,
-            n);
+          if (conv->class_type == BABL_CONVERSION_PLANAR)
+          {
+            destination_image = babl_image_from_linear (
+              destination_double_buf, BABL (BABL ((babl->fish.destination))->format.model));
+            babl_conversion_process (conv,
+              (void*)cmyka_image, (void*)destination_image, n);
+          }
+          else if (conv->class_type == BABL_CONVERSION_LINEAR)
+          {
+            babl_conversion_process (conv,
+                           cmyka_double_buf, destination_double_buf, n);
+          }
+          else
+          {
+            babl_fatal ("oops");
+          }
         }
-      else if (conv->class_type == BABL_CONVERSION_LINEAR)
+     }
+     break;
+  case KIND_RGB:
+      {
+        const Babl *destination_rgba_format =
+          babl_remodel_with_space (babl_model_from_id (BABL_RGBA),
+               destination_space);
+
+        if(BABL (babl->fish.destination)->format.model == (void*)destination_rgba_format)
         {
-          babl_conversion_process (
-            conv,
-            rgba_double_buf, destination_double_buf,
-            n);
+           destination_double_buf = rgba_double_buf;
         }
-      else babl_fatal ("oops");
-      }
-   }
+        else
+        {
+        Babl *conv =
+          assert_conversion_find (destination_rgba_format,
+             BABL (babl->fish.destination)->format.model);
+           destination_double_buf_alloc = babl_malloc (sizeof (double) * n *
+                                            BABL (babl->fish.destination)->format.model->components);
+           destination_double_buf = destination_double_buf_alloc;
 
-    convert_from_double (
-      (BablFormat *) BABL (babl->fish.destination),
-      destination_double_buf,
-      destination,
-      n
-    );
+        if (conv->class_type == BABL_CONVERSION_PLANAR)
+          {
+            destination_image = babl_image_from_linear (
+              destination_double_buf, BABL (BABL ((babl->fish.destination))->format.model));
+            babl_conversion_process (
+              conv,
+              (void*)rgba_image, (void*)destination_image,
+              n);
+          }
+        else if (conv->class_type == BABL_CONVERSION_LINEAR)
+          {
+            babl_conversion_process (
+              conv,
+              rgba_double_buf, destination_double_buf,
+              n);
+          }
+        else
+          {
+            babl_fatal ("oops");
+          }
+        }
+     }
+   break;
+  }
+  babl_mutex_unlock (babl_reference_mutex);
 
-    if (destination_double_buf_alloc)
-      babl_free (destination_double_buf_alloc);
-    if (rgba_double_buf_alloc)
-      babl_free (rgba_double_buf_alloc);
-    if (source_double_buf_alloc)
-      babl_free (source_double_buf_alloc);
-    if (source_image)
-      babl_free (source_image);
-    if (rgba_image)
-      babl_free (rgba_image);
-    if (destination_image)
-      babl_free (destination_image);
+ /* convert from double model backing target pixel format to final representation */
+  convert_from_double (
+    (BablFormat *) BABL (babl->fish.source),
+    (BablFormat *) BABL (babl->fish.destination),
+    destination_double_buf,
+    destination,
+    n
+  );
+
+  if (destination_double_buf_alloc)
+    babl_free (destination_double_buf_alloc);
+  if (rgba_double_buf_alloc)
+    babl_free (rgba_double_buf_alloc);
+  if (cmyka_double_buf_alloc)
+    babl_free (cmyka_double_buf_alloc);
+  if (source_double_buf_alloc)
+    babl_free (source_double_buf_alloc);
+  if (source_image)
+    babl_free (source_image);
+  if (rgba_image)
+    babl_free (rgba_image);
+  if (cmyka_image)
+    babl_free (cmyka_image);
+  if (destination_image)
+    babl_free (destination_image);
 }
 
 static void
@@ -870,6 +1216,7 @@ babl_fish_reference_process_float (const Babl *babl,
     Babl *conv_from_rgba;
     char dst_name[256];
 
+
     {
     char src_name[256];
     sprintf (src_name, "%s float", babl_get_name((void*)babl->fish.source->format.model));
@@ -978,7 +1325,6 @@ babl_fish_reference_process_float (const Babl *babl,
     }
 
     {
-
       if(babl_format_with_space ("RGBA float",
                    BABL (BABL ((babl->fish.destination))->format.space)) ==
          babl_format_with_space (dst_name,
@@ -1010,6 +1356,7 @@ babl_fish_reference_process_float (const Babl *babl,
     }
 
     convert_from_float (
+      (BablFormat *) BABL (babl->fish.source),
       (BablFormat *) BABL (babl->fish.destination),
       destination_float_buf,
       destination,
@@ -1070,6 +1417,13 @@ babl_fish_reference_process (const Babl *babl,
     return;
   }
 
+  if (format_has_cmyk_model (babl->fish.source) ||
+      format_has_cmyk_model (babl->fish.destination))
+  {
+    babl_fish_reference_process_double (babl, source, destination, n, data);
+    return;
+  }
+
   if (allow_float_reference == -1)
     allow_float_reference = getenv ("BABL_REFERENCE_NOFLOAT") ? 0 : 1;
 
