@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _BABL_IDS_H
@@ -25,6 +25,7 @@ enum {
   BABL_U8,
   BABL_U16,
   BABL_U32,
+  BABL_HALF,
   BABL_FLOAT,
   BABL_DOUBLE,
   BABL_HALF_FLOAT,
@@ -44,12 +45,18 @@ enum {
   BABL_RGB,
   BABL_RGBA,
   BABL_RGBA_PREMULTIPLIED,
-  BABL_GRAY_GAMMA_2_2,
-  BABL_GRAY_GAMMA_2_2_ALPHA,
-  BABL_GRAY_GAMMA_2_2_ALPHA_PREMULTIPLIED,
-  BABL_RGB_GAMMA_2_2,
-  BABL_RGBA_GAMMA_2_2,
-  BABL_RGBA_GAMMA_2_2_PREMULTIPLIED,
+  BABL_MODEL_GRAY_NONLINEAR,
+  BABL_MODEL_GRAY_NONLINEAR_ALPHA,
+  BABL_MODEL_GRAY_NONLINEAR_ALPHA_PREMULTIPLIED,
+  BABL_MODEL_GRAY_PERCEPTUAL,
+  BABL_MODEL_GRAY_PERCEPTUAL_ALPHA,
+  BABL_MODEL_GRAY_PERCEPTUAL_ALPHA_PREMULTIPLIED,
+  BABL_RGB_NONLINEAR,
+  BABL_RGBA_NONLINEAR,
+  BABL_RGBA_NONLINEAR_PREMULTIPLIED,
+  BABL_RGB_PERCEPTUAL,
+  BABL_RGBA_PERCEPTUAL,
+  BABL_RGBA_PERCEPTUAL_PREMULTIPLIED,
   BABL_YCBCR,
   BABL_YCBCR_ALPHA,
   BABL_CIE_LAB,
@@ -57,8 +64,8 @@ enum {
   BABL_MODEL_LAST_INTERNAL,
 
   BABL_COMPONENT_BASE = 10000,
-  BABL_LUMINANCE,
-  BABL_LUMINANCE_MUL_ALPHA,
+  BABL_GRAY_LINEAR,
+  BABL_GRAY_LINEAR_MUL_ALPHA,
   BABL_RED,
   BABL_GREEN,
   BABL_BLUE,
@@ -66,14 +73,21 @@ enum {
   BABL_RED_MUL_ALPHA,
   BABL_GREEN_MUL_ALPHA,
   BABL_BLUE_MUL_ALPHA,
-  BABL_LUMA,
-  BABL_LUMA_MUL_ALPHA,
-  BABL_RED_GAMMA_2_2,
-  BABL_GREEN_GAMMA_2_2,
-  BABL_BLUE_GAMMA_2_2,
-  BABL_RED_GAMMA_2_2_MUL_ALPHA,
-  BABL_GREEN_GAMMA_2_2_MUL_ALPHA,
-  BABL_BLUE_GAMMA_2_2_MUL_ALPHA,
+  BABL_GRAY_PERCEPTUAL,
+  BABL_GRAY_NONLINEAR,
+  BABL_GRAY_NONLINEAR_MUL_ALPHA,
+  BABL_RED_NONLINEAR,
+  BABL_GREEN_NONLINEAR,
+  BABL_BLUE_NONLINEAR,
+  BABL_RED_NONLINEAR_MUL_ALPHA,
+  BABL_GREEN_NONLINEAR_MUL_ALPHA,
+  BABL_BLUE_NONLINEAR_MUL_ALPHA,
+  BABL_RED_PERCEPTUAL,
+  BABL_GREEN_PERCEPTUAL,
+  BABL_BLUE_PERCEPTUAL,
+  BABL_RED_PERCEPTUAL_MUL_ALPHA,
+  BABL_GREEN_PERCEPTUAL_MUL_ALPHA,
+  BABL_BLUE_PERCEPTUAL_MUL_ALPHA,
 
 
   BABL_X,
@@ -90,6 +104,8 @@ enum {
   BABL_FORMAT_BASE = 100000,
   BABL_SRGB,
   BABL_SRGBA,
+  BABL_RGB_HALF,
+  BABL_RGBA_HALF,
   BABL_RGB_FLOAT,
   BABL_RGBA_FLOAT,
   BABL_RGBA_DOUBLE,
@@ -99,14 +115,16 @@ enum {
   BABL_RGB_U8,
   BABL_RGBA_U8,
   BABL_RGBA_U16,
+  BABL_CMYK_HALF,
   BABL_CMYK_FLOAT,
   BABL_CMYK_ALPHA_FLOAT,
+  BABL_CMYK_ALPHA_HALF,
   BABL_YCBCR411,
   BABL_YCBCR422,
   BABL_YCBCR420,
   BABL_FORMAT_LAST_INTERNAL,
 
-  BABL_PIXEL_USER_BASE,
+  BABL_PIXEL_USER_BASE
 };
 
 
