@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU Lesser General
  * Public License along with this library; if not, see
- * <http://www.gnu.org/licenses/>.
+ * <https://www.gnu.org/licenses/>.
  */
 
 #ifndef _BABL_H
@@ -80,6 +80,15 @@ const Babl * babl_component (const char *name);
  */
 const Babl * babl_model     (const char *name);
 
+/**
+ * babl_model_with_space:
+ *
+ * The models for formats also have a space in babl, try to avoid code
+ * needing to use this.
+ */
+const Babl *
+babl_model_with_space (const char *name, const Babl *space);
+
 
 /**
  * babl_space:
@@ -95,11 +104,13 @@ typedef enum {
   BABL_ICC_INTENT_PERCEPTUAL             = 0,
   BABL_ICC_INTENT_RELATIVE_COLORIMETRIC  = 1,
   BABL_ICC_INTENT_SATURATION             = 2,
-  BABL_ICC_INTENT_ABSOLUTE_COLORIMETRIC  = 3
+  BABL_ICC_INTENT_ABSOLUTE_COLORIMETRIC  = 3,
+  BABL_ICC_INTENT_PERFORMANCE            = 32
+  // black-point compensation toggle will be added if/when support exist in babl
 } BablIccIntent;
 
 /**
- * babl_icc_make_space:
+ * babl_space_from_icc:
  *
  * @icc_data: pointer to icc profile in memory
  * @icc_length: length of icc profile in bytes
@@ -121,6 +132,13 @@ typedef enum {
  * containing a message describing why the provided data does not yield a babl
  * space.
  */
+const Babl *babl_space_from_icc (const char       *icc_data,
+                                 int               icc_length,
+                                 BablIccIntent     intent,
+                                 const char      **error);
+
+
+// XXX : deprecated
 const Babl *babl_icc_make_space (const char       *icc_data,
                                  int               icc_length,
                                  BablIccIntent     intent,
@@ -152,18 +170,11 @@ char *babl_icc_get_key (const char *icc_data,
  * babl_format:
  *
  * Returns the babl object representing the color format given by
- * @name such as for example "RGB u8", "CMYK float" or "CIE Lab u16".
+ * @name such as for example "RGB u8", "CMYK float" or "CIE Lab u16",
+ * creates a format using the sRGB space, to also specify the color space
+ * and TRCs for a format, see babl_format_with_space.
  */
-const Babl * babl_format            (const char *name);
-
-/**
- * babl_format_exists:
- *
- * Returns 1 if the provided format name is known by babl or 0 if it is
- * not. Can also be used to verify that specific extension formats are
- * available (though this can also be inferred from the version of babl).
- */
-int babl_format_exists              (const char *name);
+const Babl * babl_format            (const char *encoding);
 
 /**
  * babl_format_with_space:
@@ -175,7 +186,16 @@ int babl_format_exists              (const char *name);
  * the unsuffixed version is used. If a format is passed in as space
  * the space of the format is used.
  */
-const Babl * babl_format_with_space (const char *name, const Babl *space);
+const Babl * babl_format_with_space (const char *encoding, const Babl *space);
+
+/**
+ * babl_format_exists:
+ *
+ * Returns 1 if the provided format name is known by babl or 0 if it is
+ * not. Can also be used to verify that specific extension formats are
+ * available (though this can also be inferred from the version of babl).
+ */
+int babl_format_exists              (const char *name);
 
 /*
  * babl_format_get_space:
@@ -192,14 +212,14 @@ const Babl * babl_format_get_space  (const Babl *format);
  *  destination_format, source and destination can be either strings
  *  with the names of the formats or Babl-format objects.
  */
-const Babl * babl_fish      (const void *source_format,
-                             const void *destination_format);
+const Babl * babl_fish (const void *source_format,
+                        const void *destination_format);
 
 
 /**
  * babl_fast_fish:
  *
- * Creae a faster than normal fish with specified performance (and thus
+ * Create a faster than normal fish with specified performance (and thus
  * corresponding precision tradeoff), values tolerance can hold: NULL and
  * "default", means do same as babl_fish(), other values understood in
  * increasing order of speed gain are:
@@ -263,6 +283,36 @@ int          babl_format_get_bytes_per_pixel   (const Babl *format);
  */
 const Babl * babl_format_get_model             (const Babl *format);
 
+
+
+
+enum _BablModelFlag
+{
+  BABL_MODEL_FLAG_ALPHA         = 1<<1,
+  BABL_MODEL_FLAG_PREMULTIPLIED = 1<<2,
+  BABL_MODEL_FLAG_INVERTED      = 1<<3,
+  /* BABL_MODEL_FLAG_ALPHA_ENCODED = 1<<4, NYI */
+
+  BABL_MODEL_FLAG_LINEAR        = 1<<10,
+  BABL_MODEL_FLAG_NONLINEAR     = 1<<11,
+  BABL_MODEL_FLAG_PERCEPTUAL    = 1<<12,
+
+  BABL_MODEL_FLAG_GRAY          = 1<<20,
+  BABL_MODEL_FLAG_RGB           = 1<<21,
+  /* BABL_MODEL_FLAG_SPECTRAL   = 1<<22, NYI */
+  BABL_MODEL_FLAG_CIE           = 1<<23,
+  BABL_MODEL_FLAG_CMYK          = 1<<24,
+  /* BABL_MODEL_FLAG_LUZ        = 1<<25, NYI */
+};
+
+typedef enum _BablModelFlag BablModelFlag;
+
+/* linear, nonlinear and perceptual could occupy two bits with a decidated 0,
+ * but we do not have a lack of bits in this bit pattern so leave it be.
+ */
+
+BablModelFlag babl_get_model_flags (const Babl *model);
+
 /**
  * babl_format_get_n_components:
  *
@@ -404,6 +454,20 @@ const Babl *babl_new_palette (const char  *name,
                               const Babl **format_u8,
                               const Babl **format_u8_with_alpha);
 
+/**
+ * babl_new_palette_with_space:
+ *
+ * create a new palette based format, name is optional pass in NULL to get
+ * an anonymous format. If you pass in with_alpha the format also gets
+ * an 8bit alpha channel. Returns the BablModel of the color model. If
+ * you pass in the same name the previous formats will be provided
+ * again.
+ */
+const Babl *babl_new_palette_with_space (const char  *name,
+                                         const Babl  *space,
+                                         const Babl **format_u8,
+                                         const Babl **format_u8_with_alpha);
+
 /**
  * babl_format_is_palette:
  *
@@ -431,7 +495,6 @@ void  babl_palette_set_palette (const Babl        *babl,
 void  babl_palette_reset       (const Babl        *babl);
 
 
-
 /**
  * babl_set_user_data: (skip)
  *
@@ -449,6 +512,129 @@ void   babl_set_user_data     (const Babl *babl, void *data);
  */
 void * babl_get_user_data     (const Babl *babl);
 
+typedef enum {
+  BABL_SPACE_FLAG_NONE     = 0,
+  BABL_SPACE_FLAG_EQUALIZE = 1
+} BablSpaceFlags;
+
+/**
+ * babl_space_from_chromaticities
+ *
+ * Creates a new babl-space/ RGB matrix color space definition with the
+ * specified CIE xy(Y) values for white point: wx, wy and primary
+ * chromaticities: rx,ry,gx,gy,bx,by and TRCs to be used. After registering a
+ * new babl-space it can be used with babl_space() passing its name;
+ *
+ * Internally this does the math to derive the RGBXYZ matrix as used in an ICC
+ * profile.
+ */
+const Babl * babl_space_from_chromaticities (const char *name,
+                                             double wx, double wy,
+                                             double rx, double ry,
+                                             double gx, double gy,
+                                             double bx, double by,
+                                             const Babl *trc_red,
+                                             const Babl *trc_green,
+                                             const Babl *trc_blue,
+                                             BablSpaceFlags flags);
+
+
+/**
+ * babl_trc_gamma:
+ *
+ * Creates a Babl TRC for a specific gamma value, it will be given
+ * a name that is a short string representation of the value.
+ */
+const Babl * babl_trc_gamma (double gamma);
+
+/**
+ * babl_trc:
+ *
+ * Look up a TRC by name, "sRGB" and "linear" are recognized
+ * strings in a stock babl configuration.
+ */
+const Babl * babl_trc (const char *name);
+
+/**
+ * babl_space_with_trc:
+ *
+ * Creates a variant of an existing space with different trc.
+ */
+const Babl *babl_space_with_trc (const Babl *space, const Babl *trc);
+
+/**
+ * babl_space_get:
+ *
+ * query the chromaticities of white point and primaries as well as trcs
+ * used for r g a nd b, all arguments mights be NULL.
+ */
+void babl_space_get (const Babl *space,
+                     double *xw, double *yw,
+                     double *xr, double *yr,
+                     double *xg, double *yg,
+                     double *xb, double *yb,
+                     const Babl **red_trc,
+                     const Babl **green_trc,
+                     const Babl **blue_trc);
+
+
+/**
+ * babl_model_is:
+ *
+ * return 0 if the name of the model in babl does not correspond to the provided
+ * model name.
+ */
+int babl_model_is (const Babl *babl, const char *model_name);
+
+#define babl_model_is(babl,model)  (babl&&(babl)==babl_model_with_space(model,babl))
+
+
+/**
+ * babl_space_get_icc:
+ *
+ * Return pointer to ICC profile for space note that this is
+ * the ICC profile for R'G'B', though in formats only supporting linear
+ * like EXR GEGL chooses to load this lienar data as RGB and use the sRGB
+ * TRC.
+ *
+ * @babl: a BablSpace
+ * @length: point to an integer where length of profile in bytes is stored.
+ *
+ * Returns pointer to ICC profile data.
+ */
+const char *babl_space_get_icc (const Babl *babl, int *length);
+
+/**
+ * babl_space_from_rgbxyz_matrix:
+ *
+ * Creates a new RGB matrix color space definition using a precomputed D50
+ * adapted 3x3 matrix and associated CIE XYZ whitepoint, as possibly read from
+ * an ICC profile.
+ */
+const Babl *
+babl_space_from_rgbxyz_matrix (const char *name,
+                               double wx, double wy, double wz,
+                               double rx, double gx, double bx,
+                               double ry, double gy, double by,
+                               double rz, double gz, double bz,
+                               const Babl *trc_red,
+                               const Babl *trc_green,
+                               const Babl *trc_blue);
+
+/**
+ * babl_format_get_encoding:
+ *
+ * Returns the components and data type, without space suffix.
+ */
+const char * babl_format_get_encoding (const Babl *babl);
+
+int babl_space_is_cmyk (const Babl *space);
+
+/* values below this are stored premultiplied with this value,
+ * it can also be used as a generic alpha zero epsilon in GEGL
+ *
+ */
+#define BABL_ALPHA_FLOOR (1/65536.0)
 
 #ifdef __cplusplus
 }
