@@ -42,10 +42,18 @@ conv_rgbaD_linear_rgbAD_gamma (const Babl *conversion,unsigned char *src,
    while (n--)
      {
        double alpha = fsrc[3];
+       if (alpha <= BABL_ALPHA_FLOOR)
+       {
+         if (alpha >= 0.0f)
+           alpha = BABL_ALPHA_FLOOR;
+         else if (alpha >= -BABL_ALPHA_FLOOR)
+           alpha = -BABL_ALPHA_FLOOR;
+       }
        *fdst++ = babl_trc_from_linear (trc[0], *fsrc++) * alpha;
        *fdst++ = babl_trc_from_linear (trc[1], *fsrc++) * alpha;
        *fdst++ = babl_trc_from_linear (trc[2], *fsrc++) * alpha;
-       *fdst++ = *fsrc++;
+       *fdst++ = alpha;
+       fsrc++;
      }
 }
 
@@ -65,7 +73,7 @@ conv_rgbAD_linear_rgbAD_gamma (const Babl    *conversion,
    while (n--)
      {
        double alpha = fsrc[3];
-       if (alpha < BABL_ALPHA_THRESHOLD)
+       if (alpha == 0.0)
          {
            *fdst++ = 0.0;
            *fdst++ = 0.0;
@@ -73,13 +81,6 @@ conv_rgbAD_linear_rgbAD_gamma (const Babl    *conversion,
            *fdst++ = 0.0;
            fsrc+=4;
          }
-       else if (alpha >= 1.0)
-         {
-           *fdst++ = babl_trc_from_linear (trc[0], *fsrc++) * alpha;
-           *fdst++ = babl_trc_from_linear (trc[1], *fsrc++) * alpha;
-           *fdst++ = babl_trc_from_linear (trc[2], *fsrc++) * alpha;
-           *fdst++ = *fsrc++;
-         }
        else
          {
            double alpha_recip = 1.0 / alpha;
