@@ -307,15 +307,15 @@ conv_rgbaF_linear_rgbAF_gamma (const Babl *conversion,unsigned char *src,
          *fdst++ = linear_to_gamma_2_2_lut (blue);
          *fdst++ = alpha;
        }
-       else if (alpha == 0.0)
-       {
-         *fdst++ = 0.0;
-         *fdst++ = 0.0;
-         *fdst++ = 0.0;
-         *fdst++ = 0.0;
-       }
        else
        {
+         if (alpha < BABL_ALPHA_FLOOR)
+         {
+           if (alpha >= 0.0f)
+             alpha = BABL_ALPHA_FLOOR;
+           else if (alpha >= -BABL_ALPHA_FLOOR)
+             alpha = -BABL_ALPHA_FLOOR;
+         }
          *fdst++ = linear_to_gamma_2_2_lut (red)   * alpha;
          *fdst++ = linear_to_gamma_2_2_lut (green) * alpha;
          *fdst++ = linear_to_gamma_2_2_lut (blue)  * alpha;
@@ -387,11 +387,6 @@ conv_rgbaF_linear_rgbA8_gamma (const Babl *conversion,unsigned char *src,
          *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
          *cdst++ = 0xff;
        }
-       else if (alpha <= 0.0)
-       {
-         *((uint32_t*)(cdst))=0;
-	     cdst+=4;
-       }
        else
        {
          float balpha = alpha * 0xff;
@@ -471,11 +466,6 @@ conv_rgbaF_linear_rgbA8_gamma_cairo (const Babl *conversion,unsigned char *src,
         *cdst++ = val >= 0xff ? 0xff : val <= 0 ? 0 : val;
         *cdst++ = 0xff;
       }
-      else if (alpha <= 0.0)
-      {
-        *((uint32_t*)(cdst))=0;
-        cdst+=4;
-      }
       else
       {
         float balpha = alpha * 0xff;
@@ -505,28 +495,22 @@ conv_rgbAF_linear_rgbAF_gamma (const Babl *conversion,unsigned char *src,
       float green = *fsrc++;
       float blue  = *fsrc++;
       float alpha = *fsrc++;
-       if (alpha < BABL_ALPHA_THRESHOLD)
-         {
-           *fdst++ = 0.0;
-           *fdst++ = 0.0;
-           *fdst++ = 0.0;
-           *fdst++ = 0.0;
-         }
-       else if (alpha >= 1.0)
-         {
-           *fdst++ = linear_to_gamma_2_2_lut (red);
-           *fdst++ = linear_to_gamma_2_2_lut (green);
-           *fdst++ = linear_to_gamma_2_2_lut (blue);
-           *fdst++ = *fsrc++;
-         }
-       else
-         {
-           float alpha_recip = 1.0 / alpha;
-           *fdst++ = linear_to_gamma_2_2_lut (red   * alpha_recip) * alpha;
-           *fdst++ = linear_to_gamma_2_2_lut (green * alpha_recip) * alpha;
-           *fdst++ = linear_to_gamma_2_2_lut (blue  * alpha_recip) * alpha;
-           *fdst++ = alpha;
-         }
+
+      if (alpha == 1.0)
+        {
+          *fdst++ = linear_to_gamma_2_2_lut (red);
+          *fdst++ = linear_to_gamma_2_2_lut (green);
+          *fdst++ = linear_to_gamma_2_2_lut (blue);
+          *fdst++ = *fsrc++;
+        }
+      else
+        {
+          float alpha_recip = 1.0 / alpha;
+          *fdst++ = linear_to_gamma_2_2_lut (red   * alpha_recip) * alpha;
+          *fdst++ = linear_to_gamma_2_2_lut (green * alpha_recip) * alpha;
+          *fdst++ = linear_to_gamma_2_2_lut (blue  * alpha_recip) * alpha;
+          *fdst++ = alpha;
+        }
      }
 }
 
